//
//  DataManager.swift
//  TodoList
//
//  Created by Apps4World on 12/18/21.
//

import CoreData
import Foundation

/// Main data manager to handle the todo items
class DataManager: NSObject, ObservableObject {
    /// Dynamic properties that the UI will react to
    @Published var todoItems: [TodoItem] = [TodoItem]()
    
    /// Add the Core Data container with the model name
    let container: NSPersistentContainer = NSPersistentContainer(name: "Model")
    
    /// Default init method. Load the Core Data container
    override init() {
        super.init()
        container.persistentStoreDescriptions = [
            NSPersistentStoreDescription(url: FileManager.default.containerURL(forSecurityApplicationGroupIdentifier: "group.com.a4w.todo.shared")!.appendingPathComponent("Model.sqlite"))
        ]
        container.loadPersistentStores { _, _ in }
        if todoItems.count == 0 {
            fetchAllItems { items in self.todoItems = items }
        }
    }
    
    func fetchAllItems(completion: @escaping (_ items: [TodoItem]) -> Void) {
        let fetchRequest: NSFetchRequest<Todo> = Todo.fetchRequest()
        if let results = try? container.viewContext.fetch(fetchRequest) {
            completion(results.compactMap({ entity in
                let model = TodoItem(task: entity.taskName ?? "")
                model.isCompleted = entity.isCompleted
                return model
            }))
        } else {
            completion([])
        }
    }
    
    static func updateStatus(forTask task: String) {
        let fetchRequest: NSFetchRequest<Todo> = Todo.fetchRequest()
        fetchRequest.predicate = NSPredicate(format: "taskName = %@", task)
        let container = DataManager().container
        if let entity = try? container.viewContext.fetch(fetchRequest).first {
            entity.isCompleted = !entity.isCompleted
            try? container.viewContext.save()
        }
    }
}
